from unittest.mock import (
    patch, Mock
)
from pytest import raises

from kiwi.snapshot_manager import SnapshotManager

from kiwi.exceptions import KiwiSnapshotManagerSetupError


class TestSnapshotManager:
    def test_snapshot_manager_not_implemented(self):
        with raises(KiwiSnapshotManagerSetupError):
            SnapshotManager.new(
                'foo', '/dev/device', 'root_dir', 'mountpoint', '@', [Mock()]
            )

    @patch('kiwi.snapshot_manager.snapper.SnapshotManagerSnapper')
    def test_snapshot_manager_snapper(self, mock_snapper):
        SnapshotManager.new(
            'snapper', '/dev/device', 'root_dir', 'mountpoint', '@', None
        )
        mock_snapper.assert_called_once_with(
            '/dev/device', 'root_dir', 'mountpoint', '@', None
        )



import io
from unittest.mock import (
    patch, Mock, MagicMock
)
from kiwi.repository.apk import RepositoryApk


class TestRepositoryApk:
    @patch('kiwi.repository.apk.Path.create')
    def setup(self, mock_path_create):
        root_bind = Mock()
        root_bind.root_dir = '../data'
        root_bind.shared_location = '/shared-dir'
        self.repo = RepositoryApk(root_bind)
        mock_path_create.assert_called_once_with('../data//etc/apk')

    @patch('kiwi.repository.apk.Path.create')
    def setup_method(self, cls, mock_path_create):
        self.setup()

    def test_runtime_config(self):
        assert self.repo.runtime_config()['bootstrap_repo'] == \
            self.repo.bootstrap_repo
        assert self.repo.runtime_config()['command_env'] == \
            self.repo.command_env

    def test_add_repo(self):
        with patch('builtins.open', create=True) as mock_open:
            mock_open.return_value = MagicMock(spec=io.IOBase)
            file_handle = mock_open.return_value.__enter__.return_value
            self.repo.add_repo('some', 'https://some')
            assert self.repo.bootstrap_repo == 'https://some'
            file_handle.write.assert_called_once_with('https://some\n')

    def test_setup_package_database_configuration(self):
        self.repo.setup_package_database_configuration()

    def test_import_trusted_keys(self):
        self.repo.import_trusted_keys([])

    @patch('os.path.isfile')
    @patch('os.unlink')
    def test_delete_repo(self, mock_os_unlink, mock_os_path_isfile):
        mock_os_path_isfile.return_value = True
        with patch('builtins.open', create=True) as mock_open:
            mock_open.return_value = MagicMock(spec=io.IOBase)
            file_handle = mock_open.return_value.__enter__.return_value
            self.repo.add_repo('some_A', 'https://some_A')
            self.repo.add_repo('some_B', 'https://some_B')

            mock_open.reset_mock()

            self.repo.delete_repo('some_A')
            mock_os_unlink.assert_called_once_with('../data//etc/apk/repositories')
            mock_open.assert_called_once_with('../data//etc/apk/repositories', 'a')
            file_handle.write.assert_called_once_with('https://some_B\n')

    @patch('os.unlink')
    @patch('os.path.isfile')
    def test_delete_all_repos(self, mock_os_path_isfile, mock_os_unlink):
        mock_os_path_isfile.return_value = True
        self.repo.delete_all_repos()
        mock_os_unlink.assert_called_once_with(
            '../data//etc/apk/repositories'
        )
        mock_os_path_isfile.assert_called_once_with(
            '../data//etc/apk/repositories'
        )

    def test_cleanup_unused_repos(self):
        self.repo.cleanup_unused_repos()

    def test_use_default_location(self):
        self.repo.use_default_location()

    def test_delete_repo_cache(self):
        self.repo.delete_repo_cache('unused')



from unittest.mock import (
    patch, call, Mock
)
from pytest import raises

from kiwi.chroot_manager import (
    ChrootManager, ChrootMount
)

from kiwi.exceptions import (
    KiwiCommandError,
    KiwiUmountBusyError
)


class TestChrootManager:
    @patch('kiwi.chroot_manager.MountManager')
    @patch('kiwi.chroot_manager.Command.run')
    def test_run_in_chroot_context(self, mock_run, mock_mount):
        mock_mntMngr = Mock()
        mock_mount.return_value = mock_mntMngr
        with ChrootManager(
            '/some/root', binds=[
                ChrootMount('/dev'), ChrootMount('/proc'), ChrootMount('/sys')
            ]
        ) as chroot:
            chroot.run(['cmd', 'arg1', 'arg2'])
        assert mock_mount.call_args_list == [
            call(device='/dev', mountpoint='/some/root/dev'),
            call(device='/proc', mountpoint='/some/root/proc'),
            call(device='/sys', mountpoint='/some/root/sys')
        ]
        mock_run.assert_called_once_with(
            ['chroot', '/some/root', 'cmd', 'arg1', 'arg2'],
            None, True, False, True
        )
        mock_mntMngr.bind_mount.assert_called()
        mock_mntMngr.umount.assert_called()

    @patch('kiwi.chroot_manager.MountManager')
    @patch('kiwi.chroot_manager.Command.run')
    def test_run_fails_in_chroot_context(self, mock_run, mock_mount):
        mock_mntMngr = Mock()
        mock_mount.return_value = mock_mntMngr
        mock_run.side_effect = Exception
        with ChrootManager(
            '/some/root', binds=[
                ChrootMount('/dev'), ChrootMount('/proc'), ChrootMount('/sys')
            ]
        ) as chroot:
            with raises(Exception):
                chroot.run(['cmd', 'arg1', 'arg2'])
        assert mock_mount.call_args_list == [
            call(device='/dev', mountpoint='/some/root/dev'),
            call(device='/proc', mountpoint='/some/root/proc'),
            call(device='/sys', mountpoint='/some/root/sys')
        ]
        mock_run.assert_called_once_with(
            ['chroot', '/some/root', 'cmd', 'arg1', 'arg2'],
            None, True, False, True
        )
        mock_mntMngr.bind_mount.assert_called()
        mock_mntMngr.umount.assert_called()

    @patch('kiwi.chroot_manager.MountManager')
    @patch('kiwi.chroot_manager.Command.run')
    def test_run_fails_to_enter_chroot_context(self, mock_run, mock_mount):
        mock_mntMngr = Mock()
        mock_mount.return_value = mock_mntMngr
        mock_mntMngr.bind_mount.side_effect = KiwiCommandError('mount error')
        with raises(KiwiCommandError):
            with ChrootManager(
                '/some/root', binds=[
                    ChrootMount('/dev'), ChrootMount('/proc'),
                    ChrootMount('/sys')
                ]
            ) as chroot:
                chroot.run(['cmd', 'arg1', 'arg2'])
        assert mock_mount.call_args_list == [
            call(device='/dev', mountpoint='/some/root/dev'),
            call(device='/proc', mountpoint='/some/root/proc'),
            call(device='/sys', mountpoint='/some/root/sys')
        ]
        mock_run.assert_not_called()
        mock_mntMngr.bind_mount.assert_called_once()
        mock_mntMngr.umount.assert_called()

    @patch('kiwi.chroot_manager.MountManager')
    @patch('kiwi.chroot_manager.Command.run')
    def test_run_fails_to_enter_and_clean_chroot_context(
        self, mock_run, mock_mount
    ):
        mock_mntMngr = Mock()
        mock_mount.return_value = mock_mntMngr
        mock_mntMngr.bind_mount.side_effect = KiwiCommandError('mount error')
        mock_mntMngr.umount.side_effect = KiwiUmountBusyError('umount error')
        with raises(KiwiCommandError):
            with ChrootManager(
                '/some/root', binds=[
                    ChrootMount('/dev'), ChrootMount('/proc'),
                    ChrootMount('/sys')
                ]
            ) as chroot:
                chroot.run(['cmd', 'arg1', 'arg2'])
        assert mock_mount.call_args_list == [
            call(device='/dev', mountpoint='/some/root/dev'),
            call(device='/proc', mountpoint='/some/root/proc'),
            call(device='/sys', mountpoint='/some/root/sys')
        ]
        mock_run.assert_not_called()
        mock_mntMngr.bind_mount.assert_called_once()
        mock_mntMngr.umount.assert_called()

    @patch('kiwi.chroot_manager.MountManager')
    @patch('kiwi.chroot_manager.Command.run')
    def test_run_fails_to_clean_chroot_context(
        self, mock_run, mock_mount
    ):
        mock_mntMngr = Mock()
        mock_mount.return_value = mock_mntMngr
        mock_mntMngr.umount.side_effect = KiwiUmountBusyError('umount error')
        with raises(KiwiUmountBusyError):
            with ChrootManager(
                '/some/root', binds=[
                    ChrootMount('/dev'), ChrootMount('/proc'),
                    ChrootMount('/sys')
                ]
            ) as chroot:
                chroot.run(['cmd', 'arg1', 'arg2'])
        assert mock_mount.call_args_list == [
            call(device='/dev', mountpoint='/some/root/dev'),
            call(device='/proc', mountpoint='/some/root/proc'),
            call(device='/sys', mountpoint='/some/root/sys')
        ]
        mock_run.assert_called_once_with(
            ['chroot', '/some/root', 'cmd', 'arg1', 'arg2'],
            None, True, False, True
        )
        mock_mntMngr.bind_mount.assert_called()
        mock_mntMngr.umount.assert_called()



import datetime
from unittest.mock import (
    patch, call, Mock, mock_open
)
from pytest import (
    raises
)

from lxml import etree
from xml.dom import minidom

from kiwi.snapshot_manager.snapper import SnapshotManagerSnapper

from kiwi.exceptions import (
    KiwiSnapshotManagerSetupError
)


class TestSnapshotManagerSnapper:
    def setup(self):
        self.snapper = SnapshotManagerSnapper(
            '/dev/device', 'root_dir', '/tmp/mountpoint', '@',
            {'quota_groups': True}
        )

    def setup_method(self, cls):
        self.setup()

    def test_post_init(self):
        self.snapper.post_init({'some-arg': 'some-val'})
        assert self.snapper.custom_args['some-arg'] == 'some-val'
        assert not self.snapper.custom_args['quota_groups']

    @patch('kiwi.snapshot_manager.snapper.MountManager')
    @patch('kiwi.snapshot_manager.snapper.Command.run')
    def test_create_first_snapshot_with_snapper_helper(self, mock_command, mock_mount):
        def return_snapper_version(command=None, raise_on_error=None, *args):
            mock = Mock()
            snapperCmd = ['chroot', 'snapper', '--version']
            subCmd = [element for element in command if element in snapperCmd]
            if snapperCmd == subCmd:
                mock.output = 'snapper 0.12.1'
            mock.return_code = 0
            return mock

        mock_command.side_effect = return_snapper_version

        extra_mounts = self.snapper.create_first_snapshot()
        assert mock_mount.call_args_list == [
            call(
                device='/dev/device',
                attributes={'subvol_path': '@/.snapshots', 'subvol_name': '@/.snapshots'},
                mountpoint='/tmp/mountpoint/@/.snapshots/1/snapshot/.snapshots'
            )
        ]
        assert mock_command.call_args_list == [
            call(['chroot', 'root_dir', 'snapper', '--version']),
            call(
                command=['mountpoint', '-q', 'root_dir/tmp/mountpoint'],
                raise_on_error=False
            ),
            call([
                'mount', '-n', '--bind', '/tmp/mountpoint',
                'root_dir/tmp/mountpoint'
            ]),
            call([
                'chroot', 'root_dir', '/usr/lib/snapper/installation-helper',
                '--root-prefix', '/tmp/mountpoint/@', '--step', 'filesystem'
            ], None, True, False, True),
            call(
                command=['mountpoint', '-q', 'root_dir/tmp/mountpoint'],
                raise_on_error=False
            )
        ]
        assert len(extra_mounts) == 1

    @patch('os.chmod')
    @patch('kiwi.snapshot_manager.snapper.MountManager')
    @patch('kiwi.snapshot_manager.snapper.Command.run')
    def test_create_first_snapshot_without_snapper_helper(
        self, mock_command, mock_mount, mock_os_chmod
    ):
        def return_snapper_version(command=None, raise_on_error=None, *args):
            mock = Mock()
            snapperCmd = ['chroot', 'snapper', '--version']
            subCmd = [element for element in command if element in snapperCmd]
            if snapperCmd == subCmd:
                mock.output = 'snapper 0.12.0'
            mock.return_code = 0
            return mock

        mock_command.side_effect = return_snapper_version

        extra_mounts = self.snapper.create_first_snapshot()

        mock_os_chmod.assert_called_once_with('/tmp/mountpoint/@/.snapshots', 0o700)
        assert mock_mount.call_args_list == [
            call(
                device='/dev/device',
                attributes={'subvol_path': '@/.snapshots', 'subvol_name': '@/.snapshots'},
                mountpoint='/tmp/mountpoint/@/.snapshots/1/snapshot/.snapshots'
            )
        ]
        assert mock_command.call_args_list == [
            call(['chroot', 'root_dir', 'snapper', '--version']),
            call([
                'btrfs', 'subvolume', 'create', '/tmp/mountpoint/@/.snapshots'
            ]),
            call([
                'btrfs', 'subvolume', 'create',
                '/tmp/mountpoint/@/.snapshots/1/snapshot'
            ]),
            call([
                'btrfs', 'subvolume', 'set-default',
                '/tmp/mountpoint/@/.snapshots/1/snapshot'
            ])
        ]
        assert len(extra_mounts) == 1

    @patch('os.path.exists')
    @patch('shutil.copyfile')
    @patch('kiwi.snapshot_manager.snapper.SysConfig')
    @patch('kiwi.snapshot_manager.snapper.Command.run')
    def test_setup_first_snapshot_with_snapper_helper(
        self, mock_command, mock_sysconf, mock_copy, mock_os_exists
    ):
        def return_snapper_version(command=None, raise_on_error=None, *args):
            mock = Mock()
            snapperCmd = ['chroot', 'snapper', '--version']
            subCmd = [element for element in command if element in snapperCmd]
            if snapperCmd == subCmd:
                mock.output = 'snapper 0.13.0'
            mock.return_code = 0
            return mock

        mock_command.side_effect = return_snapper_version

        item = {'SNAPPER_CONFIGS': '""'}

        def getitem(key):
            return item[key]

        def setitem(key, value):
            item[key] = value

        def contains(key):
            return key in item

        def exists(name):
            if 'snapper/configs/root' in name:
                return False
            return True

        sysconf = Mock()
        sysconf.__contains__ = Mock(side_effect=contains)
        sysconf.__getitem__ = Mock(side_effect=getitem)
        sysconf.__setitem__ = Mock(side_effect=setitem)
        mock_sysconf.return_value = sysconf
        mock_os_exists.side_effect = exists

        self.snapper.setup_first_snapshot()

        snapshot_path = '/tmp/mountpoint/@/.snapshots/1/snapshot'
        assert mock_copy.call_args_list == [
            call(
                f'{snapshot_path}/etc/snapper/config-templates/default',
                f'{snapshot_path}/etc/snapper/configs/root'
            )
        ]

        assert mock_command.call_args_list == [
            call([
                'chroot', '/tmp/mountpoint/@/.snapshots/1/snapshot',
                'snapper', '--version'
            ]),
            call(
                command=[
                    'mountpoint', '-q',
                    'root_dir/tmp/mountpoint/@/.snapshots/1/snapshot'
                ], raise_on_error=False
            ),
            call([
                'mount', '-n', '--bind',
                '/tmp/mountpoint/@/.snapshots/1/snapshot',
                'root_dir/tmp/mountpoint/@/.snapshots/1/snapshot'
            ]),
            call(
                command=[
                    'mountpoint', '-q',
                    'root_dir/tmp/mountpoint/@/.snapshots/1/snapshot/.snapshots'
                ], raise_on_error=False
            ),
            call([
                'mount', '-n', '--bind',
                '/tmp/mountpoint/@/.snapshots/1/snapshot/.snapshots',
                'root_dir/tmp/mountpoint/@/.snapshots/1/snapshot/.snapshots'
            ]),
            call([
                'chroot', 'root_dir', '/usr/lib/snapper/installation-helper',
                '--root-prefix', '/tmp/mountpoint/@/.snapshots/1/snapshot',
                '--step', 'config', '--description', 'first root filesystem'
            ], None, True, False, True),
            call(
                command=[
                    'mountpoint', '-q',
                    'root_dir/tmp/mountpoint/@/.snapshots/1/snapshot/.snapshots'
                ], raise_on_error=False
            ),
            call(
                command=[
                    'mountpoint', '-q',
                    'root_dir/tmp/mountpoint/@/.snapshots/1/snapshot'
                ], raise_on_error=False
            ),
            call(['btrfs', 'qgroup', 'create', '1/0', '/tmp/mountpoint']),
            call([
                'chroot', '/tmp/mountpoint/@/.snapshots/1/snapshot',
                'snapper', '--no-dbus', 'set-config', 'QGROUP=1/0'
            ], None, True, False, True),
            call([
                'chroot', '/tmp/mountpoint/@/.snapshots/1/snapshot',
                'snapper', '--no-dbus', 'modify', '--default', '1'
            ], None, True, False, True)
        ]

    @patch('os.path.exists')
    @patch('kiwi.snapshot_manager.snapper.Command.run')
    @patch.object(datetime, 'datetime', Mock(wraps=datetime.datetime))
    def test_setup_first_snapshot_without_snapper_helper(
        self, mock_command, mock_os_exists
    ):
        self.snapper.post_init(None)

        def return_snapper_version(command=None, raise_on_error=None, *args):
            mock = Mock()
            snapperCmd = ['chroot', 'snapper', '--version']
            subCmd = [element for element in command if element in snapperCmd]
            if snapperCmd == subCmd:
                mock.output = 'snapper 0.11.0'
            mock.return_code = 0
            return mock

        mock_command.side_effect = return_snapper_version
        mock_os_exists.return_value = True

        xml_info = etree.tostring(etree.parse(
            '../data/info.xml', etree.XMLParser(remove_blank_text=True)
        ))
        datetime.datetime.now.return_value = datetime.datetime(2016, 1, 1)

        m_open = mock_open()
        with patch('builtins.open', m_open, create=True):
            self.snapper.setup_first_snapshot()

        assert m_open.call_args_list == [
            call('/tmp/mountpoint/@/.snapshots/1/snapshot/../info.xml', 'w')
        ]
        assert m_open.return_value.write.call_args_list == [
            call(minidom.parseString(xml_info).toprettyxml(indent="    "))
        ]
        assert mock_command.call_args_list == [
            call([
                'chroot', '/tmp/mountpoint/@/.snapshots/1/snapshot',
                'snapper', '--version'
            ]),
            call([
                'chroot', '/tmp/mountpoint/@/.snapshots/1/snapshot',
                'snapper', '--no-dbus', 'modify', '--default', '1'
            ], None, True, False, True)
        ]

    @patch('os.path.exists')
    @patch('kiwi.snapshot_manager.snapper.SysConfig')
    @patch('kiwi.snapshot_manager.snapper.Command.run')
    def test_setup_first_snapshot_with_bad_sysconfig(
        self, mock_command, mock_sysconf, mock_os_exists
    ):
        def return_snapper_version(command=None, raise_on_error=None, *args):
            mock = Mock()
            snapperCmd = ['chroot', 'snapper', '--version']
            subCmd = [element for element in command if element in snapperCmd]
            if snapperCmd == subCmd:
                mock.output = 'snapper 0.13.0'
            mock.return_code = 0
            return mock

        mock_command.side_effect = return_snapper_version

        item = {'SNAPPER_CONFIGS': '"root foo"'}

        def getitem(key):
            return item[key]

        def setitem(key, value):
            item[key] = value

        def contains(key):
            return key in item

        sysconf = Mock()
        sysconf.__contains__ = Mock(side_effect=contains)
        sysconf.__getitem__ = Mock(side_effect=getitem)
        sysconf.__setitem__ = Mock(side_effect=setitem)
        mock_sysconf.return_value = sysconf
        mock_os_exists.return_value = True

        with raises(KiwiSnapshotManagerSetupError):
            self.snapper.setup_first_snapshot()

    def test_get_default_snapshot_name(self):
        default_snapshot = '/@/.snapshots/1/snapshot'
        assert self.snapper.get_default_snapshot_name() == default_snapshot



from unittest.mock import (
    patch, Mock
)
from pytest import raises

from kiwi.storage.subformat import DiskFormat

from kiwi.exceptions import KiwiDiskFormatSetupError


class TestDiskFormat:
    def setup(self):
        self.xml_state = Mock()
        self.xml_state.get_build_type_format_options.return_value = {}

    def setup_method(self, cls):
        self.setup()

    def test_format_not_implemented(self):
        with raises(KiwiDiskFormatSetupError):
            DiskFormat.new('foo', self.xml_state, 'root_dir', 'target_dir')

    def test_disk_format_vagrant_not_implemented(self):
        self.xml_state.get_build_type_vagrant_config_section = Mock(
            return_value=None
        )
        with raises(KiwiDiskFormatSetupError):
            DiskFormat.new(
                'vagrant', self.xml_state, 'root_dir', 'target_dir'
            )

    @patch('kiwi.storage.subformat.qcow2.DiskFormatQcow2')
    def test_disk_format_qcow2(self, mock_qcow2):
        DiskFormat.new('qcow2', self.xml_state, 'root_dir', 'target_dir')
        mock_qcow2.assert_called_once_with(
            self.xml_state, 'root_dir', 'target_dir', {}
        )

    @patch('kiwi.storage.subformat.vdi.DiskFormatVdi')
    def test_disk_format_vdi(self, mock_vdi):
        DiskFormat.new('vdi', self.xml_state, 'root_dir', 'target_dir')
        mock_vdi.assert_called_once_with(
            self.xml_state, 'root_dir', 'target_dir', {}
        )

    @patch('kiwi.storage.subformat.vhd.DiskFormatVhd')
    def test_disk_format_vhd(self, mock_vhd):
        DiskFormat.new('vhd', self.xml_state, 'root_dir', 'target_dir')
        mock_vhd.assert_called_once_with(
            self.xml_state, 'root_dir', 'target_dir', {}
        )

    @patch('kiwi.storage.subformat.vhdx.DiskFormatVhdx')
    def test_disk_format_vhdx(self, mock_vhdx):
        DiskFormat.new('vhdx', self.xml_state, 'root_dir', 'target_dir')
        mock_vhdx.assert_called_once_with(
            self.xml_state, 'root_dir', 'target_dir', {}
        )

    @patch('kiwi.storage.subformat.vhdfixed.DiskFormatVhdFixed')
    def test_disk_format_vhdfixed(self, mock_vhdfixed):
        self.xml_state.build_type.get_vhdfixedtag = Mock(
            return_value='disk-tag'
        )
        DiskFormat.new('vhd-fixed', self.xml_state, 'root_dir', 'target_dir')
        mock_vhdfixed.assert_called_once_with(
            self.xml_state, 'root_dir', 'target_dir', {'--tag': 'disk-tag'}
        )

    @patch('kiwi.storage.subformat.gce.DiskFormatGce')
    def test_disk_format_gce(self, mock_gce):
        self.xml_state.build_type.get_gcelicense = Mock(
            return_value='gce_license_tag'
        )
        DiskFormat.new('gce', self.xml_state, 'root_dir', 'target_dir')
        mock_gce.assert_called_once_with(
            self.xml_state, 'root_dir', 'target_dir',
            {'--tag': 'gce_license_tag'}
        )

    @patch('kiwi.storage.subformat.vmdk.DiskFormatVmdk')
    def test_disk_format_vmdk(self, mock_vmdk):
        vmdisk = Mock()
        vmdisk.get_controller = Mock(
            return_value='controller'
        )
        vmdisk.get_diskmode = Mock(
            return_value='disk-mode'
        )
        self.xml_state.get_build_type_vmdisk_section = Mock(
            return_value=vmdisk
        )
        DiskFormat.new('vmdk', self.xml_state, 'root_dir', 'target_dir')
        mock_vmdk.assert_called_once_with(
            self.xml_state, 'root_dir', 'target_dir',
            {'adapter_type=controller': None, 'subformat=disk-mode': None}
        )

    @patch('kiwi.storage.subformat.ova.DiskFormatOva')
    def test_disk_format_ova(self, mock_ova):
        vmdisk = Mock()
        vmdisk.get_controller = Mock(
            return_value='controller'
        )
        vmdisk.get_diskmode = Mock(
            return_value='disk-mode'
        )
        self.xml_state.get_build_type_vmdisk_section = Mock(
            return_value=vmdisk
        )
        DiskFormat.new('ova', self.xml_state, 'root_dir', 'target_dir')
        mock_ova.assert_called_once_with(
            self.xml_state, 'root_dir', 'target_dir',
            {'adapter_type=controller': None, 'subformat=disk-mode': None}
        )

    @patch('kiwi.storage.subformat.oci.DiskFormatOci')
    def test_disk_format_oci(self, mock_oci):
        DiskFormat.new(
            'oci:raw', self.xml_state, 'root_dir', 'target_dir'
        )
        mock_oci.assert_called_once_with(
            self.xml_state, 'root_dir', 'target_dir',
            {'base_format': 'raw', 'container_format': 'oci'}
        )
        mock_oci.reset_mock()
        DiskFormat.new(
            'docker:raw', self.xml_state, 'root_dir', 'target_dir'
        )
        mock_oci.assert_called_once_with(
            self.xml_state, 'root_dir', 'target_dir',
            {'base_format': 'raw', 'container_format': 'docker'}
        )

    @patch('kiwi.storage.subformat.vagrant_virtualbox.DiskFormatVagrantVirtualBox')
    @patch('kiwi.storage.subformat.vagrant_libvirt.DiskFormatVagrantLibVirt')
    def test_disk_format_vagrant_libvirt(
        self, mock_vagrant_libvirt, mock_vagrant_virt_box
    ):
        for provider_name, provider_mock in (
                ('libvirt', mock_vagrant_libvirt),
                ('virtualbox', mock_vagrant_virt_box)
        ):
            vagrant_config = Mock()
            vagrant_config.get_provider = Mock(
                return_value=provider_name
            )
            self.xml_state.get_build_type_vagrant_config_section = Mock(
                return_value=vagrant_config
            )
            DiskFormat.new(
                'vagrant', self.xml_state, 'root_dir', 'target_dir'
            )
            provider_mock.assert_called_once_with(
                self.xml_state, 'root_dir', 'target_dir',
                {'vagrantconfig': vagrant_config}
            )

    @patch('kiwi.storage.subformat.base.DiskFormatBase')
    def test_disk_format_base(self, mock_base):
        DiskFormat.new('raw', self.xml_state, 'root_dir', 'target_dir')
        mock_base.assert_called_once_with(
            self.xml_state, 'root_dir', 'target_dir', {}
        )



# Copyright (c) 2022 Marcus Schäfer.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
# project
from kiwi.runtime_config import RuntimeConfig
from kiwi.utils.temporary import Temporary
from kiwi.command import Command

from kiwi.exceptions import KiwiCredentialsError


class Signature:
    """
    **Create signatures**
    """
    def __init__(self, filepath: str) -> None:
        """
        Construct new Signature object

        :param str filepath: file path name
        """
        self.filepath = filepath

    def sign(self) -> None:
        """
        Create an openssl based signature from the given file name
        and attach it at the end of that filename. This method requires
        access to a private key for signing. The path to the private
        key is read from the kiwi runtime config file from the
        following section:

        credentials:
          - verification_metadata_signing_key_file: /path/to/pkey
        """
        runtime_config = RuntimeConfig()
        signing_key_file = runtime_config.\
            get_credentials_verification_metadata_signing_key_file()
        if not signing_key_file:
            raise KiwiCredentialsError(
                '{0} not configured in runtime config'.format(
                    'verification_metadata_signing_key_file'
                )
            )
        signature_file = Temporary().new_file()
        Command.run(
            [
                'openssl', 'dgst', '-sha256',
                '-sigopt', 'rsa_padding_mode:pss',
                '-sigopt', 'rsa_pss_saltlen:-1',
                '-sigopt', 'rsa_mgf1_md:sha256',
                '-sign', signing_key_file,
                '-out', signature_file.name,
                self.filepath
            ]
        )
        with open(signature_file.name, 'rb') as sig_fd:
            signature = sig_fd.read()
            with open(self.filepath, 'ab') as get_signed:
                get_signed.write(signature)



from unittest.mock import patch

import unittest.mock as mock

from kiwi.filesystem.btrfs import FileSystemBtrfs


class TestFileSystemBtrfs:
    @patch('os.path.exists')
    def setup(self, mock_exists):
        mock_exists.return_value = True
        provider = mock.Mock()
        provider.get_device = mock.Mock(
            return_value='/dev/foo'
        )
        self.btrfs = FileSystemBtrfs(provider, 'root_dir')
        self.btrfs.setup_mountpoint = mock.Mock(
            return_value='some-mount-point'
        )

    @patch('os.path.exists')
    def setup_method(self, cls, mock_exists):
        self.setup()

    @patch('kiwi.filesystem.btrfs.Command.run')
    def test_create_on_device(self, mock_command):
        with patch.dict('os.environ', {'SOURCE_DATE_EPOCH': '0'}):
            self.btrfs.create_on_device('label', 100, uuid='uuid')
            call = mock_command.call_args_list[0]
            assert mock_command.call_args_list[0] == call(
                [
                    'mkfs.btrfs', '-L', 'label', '-U', 'uuid',
                    '--byte-count', '102400', '/dev/foo'
                ]
            )
            mock_command.reset_mock()
            self.btrfs.create_on_device('label', 100)
            assert mock_command.call_args_list[0] == call(
                [
                    'mkfs.btrfs',
                    '-L', 'label',
                    '-U', '2453562e-7073-2098-d091-fd7a04dc5434',
                    '--byte-count', '102400',
                    '/dev/foo'
                ]
            )

    @patch('kiwi.filesystem.btrfs.Command.run')
    def test_set_uuid(self, mock_command):
        self.btrfs.set_uuid()
        mock_command.assert_called_once_with(
            ['btrfstune', '-f', '-u', '/dev/foo']
        )



# Copyright (c) 2024 SUSE Software Solutions Germany GmbH
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
import copy
import logging
import shutil
from string import Template
from typing import Dict

# project
from kiwi.path import Path
from kiwi.boot.image.base import BootImageBase
from kiwi.bootloader.template.zipl import BootLoaderTemplateZipl
from kiwi.bootloader.config.bootloader_spec_base import BootLoaderSpecBase
from kiwi.command import Command
from kiwi.utils.temporary import Temporary

from kiwi.exceptions import (
    KiwiTemplateError,
    KiwiBootLoaderTargetError,
    KiwiDiskGeometryError
)

log = logging.getLogger('kiwi')


class BootLoaderZipl(BootLoaderSpecBase):
    def create_loader_image(self, target: str) -> None:
        """
        Nothing to be done here for zipl

        :param str target: unused
        """
        pass  # pragma: nocover

    def setup_loader(self, target: str) -> None:
        """
        Setup temporary zipl config and install zipl for supported targets.
        Please note we are not touching the main zipl.conf file provided
        by the distributors

        :param str target:
            target identifier, one of disk, live(iso) or install(iso)
            Currently only the disk identifier is supported
        """
        if target != self.target.disk:
            raise KiwiBootLoaderTargetError(
                'zipl is only supported with the disk image target'
            )
        boot_path = self.get_boot_path()
        boot_options = self.custom_args['boot_options']
        self._mount_system(
            boot_options.get('device_map'),
            boot_options.get('root_device'),
            boot_options.get('boot_device'),
            boot_options.get('efi_device'),
            boot_options.get('system_volumes'),
            boot_options.get('system_root_volume'),
            boot_options.get('system_partitions')
        )
        root_dir = self.root_mount.mountpoint
        kernel_info = BootImageBase(
            self.xml_state, root_dir, root_dir
        ).get_boot_names()
        self.custom_args['secure_linux'] = False
        self.custom_args['kernel'] = \
            f'{boot_path}/{os.path.basename(kernel_info.kernel_filename)}'
        self.custom_args['initrd'] = \
            f'{boot_path}/{kernel_info.initrd_name}'

        host_key_certificates = self.xml_state.get_host_key_certificates()
        cc_boot_image = f'{self.custom_args["kernel"]}.cc'
        if host_key_certificates:
            with Temporary(
                path=self.root_mount.mountpoint, prefix='kiwi_zipl_parmfile_'
            ).new_file() as hkd_parm_file:
                with open(hkd_parm_file.name, 'w') as parm_file:
                    parm_file.write(f'{self.cmdline}{os.linesep}')
                genprotimg_init = [
                    'chroot', root_dir, 'genprotimg',
                    '--offline', '--verbose',
                    '-o', cc_boot_image,
                    '-i', self.custom_args['kernel'],
                    '-r', self.custom_args['initrd'],
                    '-p', hkd_parm_file.name.replace(root_dir, '')
                ]
                # verify all host key documents individually and call
                # genprotimg with --no-verify afterwards. This is done
                # because genprotimg does not support verification with
                # multiple signing keys
                for host_key_certificate in host_key_certificates:
                    genprotimg_host_key_check = copy.deepcopy(genprotimg_init)
                    genprotimg_host_key_check.append('--cert')
                    genprotimg_host_key_check.append(
                        host_key_certificates[0]['hkd_ca_cert']
                    )
                    genprotimg_host_key_check.append('--cert')
                    genprotimg_host_key_check.append(
                        host_key_certificate['hkd_sign_cert']
                    )
                    for host_key in host_key_certificate.get('hkd_cert'):
                        genprotimg_host_key_check.append('-k')
                        genprotimg_host_key_check.append(host_key)
                    for host_crl in host_key_certificate.get('hkd_revocation_list'):
                        genprotimg_host_key_check.append('--crl')
                        genprotimg_host_key_check.append(host_crl)
                    log.info(
                        'Checking HKDs for signing cert: {}'.format(
                            host_key_certificate['hkd_sign_cert']
                        )
                    )
                    Command.run(genprotimg_host_key_check)
                    os.unlink(f'{root_dir}/{cc_boot_image}')

                # generate secure execution image
                genprotimg = genprotimg_init
                genprotimg.append('--no-verify')
                for host_key_certificate in host_key_certificates:
                    for host_key in host_key_certificate.get('hkd_cert'):
                        genprotimg.append('-k')
                        genprotimg.append(host_key)
                    for host_crl in host_key_certificate.get('hkd_revocation_list'):
                        genprotimg.append('--crl')
                        genprotimg.append(host_crl)
                Command.run(genprotimg)

                # extract secure execution header
                secure_execution_header = 'secure_execution_header.bin'
                Command.run(
                    [
                        'chroot', root_dir, 'pvextract-hdr',
                        '-o', secure_execution_header,
                        cc_boot_image
                    ]
                )
                shutil.move(
                    os.sep.join([root_dir, secure_execution_header]),
                    os.sep.join(
                        [
                            self.custom_args['target_dir'],
                            '{0}.{1}-{2}.{3}'.format(
                                self.xml_state.xml_data.get_name(),
                                self.arch,
                                self.xml_state.get_image_version(),
                                secure_execution_header
                            )
                        ]
                    )
                )

                # generate attestation request binary and response key
                attestation_request = 'attestation_request.bin'
                attestation_response_key = 'attestation_response.key'
                pvattest = [
                    'chroot', root_dir, 'pvattest', 'create',
                    '--no-verify', '--verbose',
                    '-o', attestation_request,
                    '--arpk', attestation_response_key
                ]
                for host_key_certificate in host_key_certificates:
                    for host_key in host_key_certificate.get('hkd_cert'):
                        pvattest.append('-k')
                        pvattest.append(host_key)
                Command.run(pvattest)
                shutil.move(
                    os.sep.join([root_dir, attestation_request]),
                    os.sep.join(
                        [
                            self.custom_args['target_dir'],
                            '{0}.{1}-{2}.{3}'.format(
                                self.xml_state.xml_data.get_name(),
                                self.arch,
                                self.xml_state.get_image_version(),
                                attestation_request
                            )
                        ]
                    )
                )
                shutil.move(
                    os.sep.join([root_dir, attestation_response_key]),
                    os.sep.join(
                        [
                            self.custom_args['target_dir'],
                            '{0}.{1}-{2}.{3}'.format(
                                self.xml_state.xml_data.get_name(),
                                self.arch,
                                self.xml_state.get_image_version(),
                                attestation_response_key
                            )
                        ]
                    )
                )

            self.custom_args['secure_linux'] = True
            self.custom_args['secure_image_file'] = cc_boot_image
            os.unlink(f'{root_dir}/{self.custom_args["kernel"]}')
            os.unlink(f'{root_dir}/{self.custom_args["initrd"]}')
            self.custom_args['kernel'] = ''
            self.custom_args['initrd'] = ''

        with Temporary(
            path=self.root_mount.mountpoint, prefix='kiwi_zipl.conf_'
        ).new_file() as runtime_zipl_config_file:
            BootLoaderZipl._write_config_file(
                BootLoaderTemplateZipl().get_loader_template(),
                runtime_zipl_config_file.name,
                self._get_template_parameters()
            )
            self.set_loader_entry(
                self.root_mount.mountpoint, self.target.disk
            )
            self._install_zipl(root_dir, runtime_zipl_config_file.name)

    def set_loader_entry(self, root_dir: str, target: str) -> None:
        """
        Setup/update loader entries of the form
        {boot_path}/loader/entries/{get_entry_name}

        :param str target:
            target identifier, one of disk, live(iso) or install(iso)
        """
        entry_name = self.get_entry_name()
        BootLoaderZipl._write_config_file(
            BootLoaderTemplateZipl().get_entry_template(
                self.custom_args['secure_linux']
            ),
            root_dir + f'{self.entries_dir}/{entry_name}',
            self._get_template_parameters(entry_name)
        )

    def _install_zipl(self, root_dir: str, zipl_config: str) -> None:
        """
        Install zipl on target
        """
        zipl = [
            'chroot', root_dir, 'zipl',
            '--noninteractive',
            '--config', zipl_config.replace(root_dir, ''),
            '--blsdir', self.entries_dir,
            '--verbose'
        ]
        self.sys_mount.umount()
        Command.run(zipl)

    def _get_template_parameters(
        self, default_entry: str = ''
    ) -> Dict[str, str]:
        disk_type = self.disk_type or 'SCSI'
        disk_type = disk_type if disk_type != 'GPT' else 'SCSI'
        blocksize = self.disk_blocksize or 512
        unsupported_for_target_geometry = ['FBA', 'SCSI']
        targetbase = f'targetbase={self.custom_args.get("targetbase")}'
        geometry = ''
        if disk_type not in unsupported_for_target_geometry:
            geometry = f'targetgeometry={self._get_disk_geometry()}'
        return {
            'secure_image_file': self.custom_args.get('secure_image_file') or '',
            'kernel_file': self.custom_args['kernel'] or 'vmlinuz',
            'initrd_file': self.custom_args['initrd'] or 'initrd',
            'boot_options': self.cmdline,
            'boot_timeout': self.timeout,
            'bootpath': self.get_boot_path(),
            'targetbase': targetbase,
            'targettype': disk_type,
            'targetblocksize': format(blocksize),
            'targetoffset': self._get_partition_start(),
            'targetgeometry': geometry,
            'title': self.get_menu_entry_title(),
            'default_entry': default_entry
        }

    def _get_disk_geometry(self) -> str:
        target_table_type = self.firmware.get_partition_table_type()
        disk_device = self.custom_args['targetbase']
        disk_geometry = ''
        if target_table_type == 'dasd':
            disk_geometry = '{0},{1},{2}'.format(
                self._get_dasd_disk_geometry_element(
                    disk_device, 'cylinders'
                ),
                self._get_dasd_disk_geometry_element(
                    disk_device, 'tracks per cylinder'
                ),
                self._get_dasd_disk_geometry_element(
                    disk_device, 'blocks per track'
                )
            )
        return disk_geometry

    def _get_partition_start(self) -> str:
        target_table_type = self.firmware.get_partition_table_type()
        disk_device = self.custom_args['targetbase']
        if target_table_type == 'dasd':
            blocks = self._get_dasd_disk_geometry_element(
                disk_device, 'blocks per track'
            )
            fdasd_command = [
                'fdasd', '-f', '-s', '-p', disk_device,
                '|', 'grep', '"^ "',
                '|', 'head', '-n', '1',
                '|', 'tr', '-s', '" "'
            ]
            fdasd_call = Command.run(
                ['bash', '-c', ' '.join(fdasd_command)]
            )
            fdasd_output = fdasd_call.output
            try:
                start_track = int(fdasd_output.split(' ')[2].lstrip())
            except Exception:
                raise KiwiDiskGeometryError(
                    f'unknown partition format: {fdasd_output}'
                )
            return '{0}'.format(start_track * blocks)
        else:
            sfdisk_command = ' '.join(
                [
                    'sfdisk', '--dump', disk_device,
                    '|', 'grep', '"1 :"',
                    '|', 'cut', '-f1', '-d,',
                    '|', 'cut', '-f2', '-d='
                ]
            )
            return Command.run(
                ['bash', '-c', sfdisk_command]
            ).output.strip()

    def _get_dasd_disk_geometry_element(self, disk_device, search) -> int:
        fdasd = ['fdasd', '-f', '-p', disk_device]
        bash_command = fdasd + ['|', 'grep', '"' + search + '"']
        fdasd_call = Command.run(
            ['bash', '-c', ' '.join(bash_command)]
        )
        fdasd_output = fdasd_call.output
        try:
            return int(fdasd_output.split(':')[1].lstrip())
        except Exception:
            raise KiwiDiskGeometryError(
                f'unknown format for disk geometry: {fdasd_output}'
            )

    @staticmethod
    def _write_config_file(
        template: Template, filename: str, parameters: Dict[str, str]
    ) -> None:
        try:
            config_data = template.substitute(parameters)
            Path.create(os.path.dirname(filename))
            with open(filename, 'w') as config:
                config.write(config_data)
        except Exception as issue:
            raise KiwiTemplateError(
                '{0}: {1}'.format(type(issue).__name__, issue)
            )



from unittest.mock import (
    Mock, patch
)

from kiwi.utils.block import BlockID


class TestBlockID:
    def setup(self):
        self.blkid = BlockID('device')

    def setup_method(self, cls):
        self.setup()

    @patch('kiwi.utils.block.Command.run')
    def test_setup_with_uuid_format(self, mock_command):
        BlockID('UUID=uuid')
        mock_command.assert_called_once_with(
            ['blkid', '--uuid', 'uuid']
        )

    @patch('kiwi.utils.block.Command.run')
    def test_get_blkid(self, mock_command):
        self.blkid.get_blkid('LABEL')
        mock_command.assert_called_once_with(
            ['blkid', 'device', '-s', 'LABEL', '-o', 'value'],
            raise_on_error=False
        )

    @patch('kiwi.utils.block.BlockID.get_blkid')
    def test_get_filesystem(self, mock_get_blkid):
        self.blkid.get_filesystem()
        mock_get_blkid.assert_called_once_with('TYPE')

    @patch('kiwi.utils.block.BlockID.get_blkid')
    def test_get_label(self, mock_get_blkid):
        self.blkid.get_label()
        mock_get_blkid.assert_called_once_with('LABEL')

    @patch('kiwi.utils.block.BlockID.get_blkid')
    def test_get_uuid(self, mock_get_blkid):
        self.blkid.get_uuid()
        mock_get_blkid.assert_called_once_with('UUID')

    @patch('kiwi.utils.block.BlockID.get_blkid')
    def test_get_ptuuid(self, mock_get_blkid):
        self.blkid.get_ptuuid()
        mock_get_blkid.assert_called_once_with('PTUUID')

    @patch('kiwi.utils.block.Command.run')
    def test_get_partition_count(self, mock_Command_run):
        lsblk_call = Mock()
        lsblk_call.output = "NAME TYPE\nsda disk\nsda4 part \nsda3 part"
        mock_Command_run.return_value = lsblk_call
        assert self.blkid.get_partition_count() == 2



from pytest import raises
from unittest.mock import (
    patch, Mock
)

import kiwi

from kiwi.defaults import Defaults
from kiwi.storage.subformat.oci import DiskFormatOci
from kiwi.exceptions import KiwiContainerBuilderError
from kiwi.system.uri import Uri


class TestDiskFormatOci:
    def setup(self):
        Defaults.set_platform_name('x86_64')
        xml_data = Mock()
        xml_data.get_name = Mock(
            return_value='some-disk-image'
        )
        self.xml_state = Mock()
        self.uri = Uri(
            'docker-archive:/home/ms/alpine.tar', repo_type='container'
        )
        self.xml_state.xml_data = xml_data
        self.xml_state.get_derived_from_image_uri = Mock(
            return_value=[self.uri]
        )
        self.xml_state.get_image_version = Mock(
            return_value='1.2.3'
        )
        self.runtime_config = Mock()
        kiwi.storage.subformat.base.RuntimeConfig = Mock(
            return_value=self.runtime_config
        )
        self.disk_format = DiskFormatOci(
            self.xml_state, 'root_dir', 'target_dir'
        )
        self.disk_format.base_format = 'raw'

    def setup_method(self, cls):
        self.setup()

    def test_post_init(self):
        self.disk_format.post_init({'base_format': 'raw'})
        assert self.disk_format.base_container_uris == [self.uri]
        assert self.disk_format.base_format == 'raw'

    @patch('kiwi.storage.subformat.oci.Temporary.new_dir')
    @patch('kiwi.storage.subformat.oci.Command.run')
    @patch('kiwi.storage.subformat.oci.Defaults')
    @patch('kiwi.storage.subformat.oci.ContainerImage')
    @patch('kiwi.storage.subformat.oci.DiskFormat')
    @patch('kiwi.storage.subformat.oci.Path')
    @patch('kiwi.storage.subformat.oci.Result')
    @patch('os.chown')
    @patch('os.stat')
    def test_create_image_format_empty_root(
        self,
        mock_stat,
        mock_chown,
        mock_Result,
        mock_Path,
        mock_DiskFormat,
        mock_ContainerImage,
        mock_Defaults,
        mock_Command_run,
        mock_Temporary_new_dir
    ):
        mock_Temporary_new_dir.return_value.name = 'root_dir'
        result = Mock()
        result.get_name_for_pattern.return_value = 'some_custom_name'
        mock_Result.return_value = result
        self.disk_format.bundle_format = 'some'
        self.disk_format.base_container_uris = []
        mock_Defaults.get_imported_root_image.return_value \
            = 'some-base'

        # test raw format
        self.disk_format.create_image_format()

        mock_Path.assert_called_once_with('root_dir/disk')
        mock_Command_run.assert_called_once_with(
            [
                'cp', '-a',
                'target_dir/some-disk-image.x86_64-1.2.3.raw',
                'root_dir/disk/some_custom_name.raw'
            ]
        )
        mock_ContainerImage.new.assert_called_once_with(
            'docker', 'root_dir',
            self.xml_state.get_container_config.return_value
        )
        mock_ContainerImage.new.return_value.create.assert_called_once_with(
            'target_dir/some-disk-image.x86_64-1.2.3.docker.tar',
            '', True, True
        )

    @patch('kiwi.storage.subformat.oci.Command.run')
    @patch('kiwi.storage.subformat.oci.Temporary.new_dir')
    @patch('kiwi.storage.subformat.oci.RootImport')
    @patch('kiwi.storage.subformat.oci.Defaults')
    @patch('kiwi.storage.subformat.oci.Checksum')
    @patch('kiwi.storage.subformat.oci.ContainerImage')
    @patch('kiwi.storage.subformat.oci.RootInit')
    @patch('kiwi.storage.subformat.oci.DiskFormat')
    @patch('kiwi.storage.subformat.oci.Path')
    @patch('kiwi.storage.subformat.oci.Result')
    @patch('os.chown')
    @patch('os.stat')
    def test_create_image_format_derived_from(
        self,
        mock_stat,
        mock_chown,
        mock_Result,
        mock_Path,
        mock_DiskFormat,
        mock_RootInit,
        mock_ContainerImage,
        mock_Checksum,
        mock_Defaults,
        mock_RootImport,
        mock_Temporary_new_dir,
        mock_Command_run
    ):
        result = Mock()
        result.get_name_for_pattern.return_value = 'some_custom_name'
        mock_Result.return_value = result
        self.disk_format.bundle_format = 'some'
        mock_Defaults.get_imported_root_image.return_value \
            = 'some-base'
        tmpdir = Mock()
        tmpdir.name = 'tmpdir'
        mock_Temporary_new_dir.return_value = tmpdir

        # test raw format
        self.disk_format.create_image_format()

        mock_RootInit.assert_called_once_with(
            'tmpdir', allow_existing=True
        )
        mock_RootInit.return_value.create.assert_called_once_with()
        mock_RootImport.new.assert_called_once_with(
            'tmpdir', [self.uri], 'docker'
        )
        mock_RootImport.new.return_value.sync_data.assert_called_once_with()
        mock_Path.assert_called_once_with('tmpdir/disk')
        mock_Command_run.assert_called_once_with(
            [
                'cp', '-a',
                'target_dir/some-disk-image.x86_64-1.2.3.raw',
                'tmpdir/disk/some_custom_name.raw'
            ]
        )
        mock_Checksum.return_value.matches.assert_called_once_with(
            mock_Checksum.return_value.sha256.return_value, 'some-base.sha256'
        )
        mock_ContainerImage.new.assert_called_once_with(
            'docker', 'tmpdir', self.xml_state.get_container_config.return_value
        )
        mock_ContainerImage.new.return_value.create.assert_called_once_with(
            'target_dir/some-disk-image.x86_64-1.2.3.docker.tar',
            'some-base', True, True
        )

        # test qcow2 format
        mock_Command_run.reset_mock()
        self.disk_format.base_format = 'qcow2'
        self.disk_format.create_image_format()
        mock_DiskFormat.new.assert_called_once_with(
            'qcow2', self.xml_state, 'root_dir', 'target_dir'
        )
        disk_format = mock_DiskFormat.new.return_value.__enter__.return_value
        disk_format.create_image_format.assert_called_once_with()
        mock_Command_run.assert_called_once_with(
            [
                'cp', '-a',
                disk_format.get_target_file_path_for_format.return_value,
                'tmpdir/disk/some_custom_name.qcow2'
            ]
        )

        # test checksum failed for base container
        mock_Checksum.return_value.matches.reset_mock()
        mock_Checksum.return_value.matches.return_value = False
        with raises(KiwiContainerBuilderError):
            self.disk_format.create_image_format()

    def test_store_to_result(self):
        result = Mock()
        self.disk_format.store_to_result(result)
        result.add.assert_called_once_with(
            compress=False,
            filename='target_dir/some-disk-image.x86_64-1.2.3.docker.tar',
            key='disk_format_image',
            shasum=True,
            use_for_bundle=True
        )



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#


class BootLoaderInstallBase:
    """
    **Base class for bootloader installation on device**

    :param string root_dir: root directory path name
    :param object device_provider: instance of :class:`DeviceProvider`
    :param dict custom_args: custom arguments dictionary
    """
    def __init__(
        self, xml_state, root_dir, device_provider, custom_args=None
    ):
        self.xml_state = xml_state
        self.root_dir = root_dir
        self.device_provider = device_provider

        self.device = self.device_provider.get_device()

        self.post_init(custom_args)

    def post_init(self, custom_args):
        """
        Post initialization method

        Store custom arguments by default

        :param dict custom_args: custom bootloader arguments
        """
        self.custom_args = custom_args

    def install_required(self):
        """
        Check if bootloader needs to be installed

        Implementation in specialized bootloader install class required
        """
        raise NotImplementedError

    def install(self):
        """
        Install bootloader on self.device

        Implementation in specialized bootloader install class required
        """
        raise NotImplementedError

    def secure_boot_install(self):
        """
        Run shim-install in self.device for secure boots

        Implementation in specialized bootloader install class required
        """
        raise NotImplementedError

    def set_disk_password(self, password: str):
        """
        Include crypto mount credentials

        Implementation in specialized bootloader install class optional
        """
        pass



# Copyright (c) 2015 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
from typing import Optional, Any, List
from collections.abc import Iterable

# project
from kiwi.utils.temporary import Temporary
from kiwi.command import Command
from kiwi.defaults import Defaults

from kiwi.exceptions import KiwiShellVariableValueError


class Shell:
    """
    **Special character handling for shell evaluated code**
    """
    @staticmethod
    def quote(message: str) -> str:
        """
        Quote characters which have a special meaning for bash
        but should be used as normal characters. actually I had
        planned to use pipes.quote but it does not quote as I
        had expected it. e.g 'name_wit_a_$' does not quote the $
        so we do it on our own for the scope of kiwi

        :param str message: message text

        :return: quoted text

        :rtype: str
        """
        # \\ quoting must be first in the list
        quote_characters = ['\\', '$', '"', '`', '!']
        for quote in quote_characters:
            message = message.replace(quote, '\\' + quote)
        return message

    @staticmethod
    def quote_key_value_file(filename: str) -> List[str]:
        """
        Quote given input file which has to be of the form
        key=value to be able to become sourced by the shell

        :param str filename: file path name

        :return: list of quoted text

        :rtype: List[str]
        """
        temp_copy = Temporary().new_file()
        Command.run(['cp', filename, temp_copy.name])
        Shell.run_common_function('baseQuoteFile', [temp_copy.name])
        with open(temp_copy.name) as quoted:
            return quoted.read().splitlines()

    @staticmethod
    def run_common_function(name: str, parameters: List[str]) -> None:
        """
        Run a function implemented in config/functions.sh

        :param str name: function name
        :param list parameters: function arguments
        """
        Command.run(
            [
                'bash', '-c',
                'source ' + ''.join(
                    [
                        Defaults.get_common_functions_file(),
                        '; ', name, ' ', ' '.join(parameters)
                    ]
                )
            ]
        )

    @staticmethod
    def format_to_variable_value(value: Optional[Any]) -> str:
        """
        Format given variable value to return a string value
        representation that can be sourced by shell scripts.
        If the provided value is not representable as a string
        (list, dict, tuple etc) an exception is raised

        :param any value: a python variable

        :raises KiwiShellVariableValueError: if value is an iterable

        :return: string value representation

        :rtype: str
        """
        if value is None:
            return ''
        if isinstance(value, bool):
            return format(value).lower()
        if isinstance(value, str):
            return value
        if isinstance(value, bytes):
            return format(value.decode())
        if isinstance(value, Iterable):
            # we will have a hard time to turn an iterable (list, dict ...)
            # into a useful string
            raise KiwiShellVariableValueError(
                'Value cannot be {0}'.format(type(value))
            )
        return format(value)



from kiwi.bootloader.template.zipl import BootLoaderTemplateZipl


class TestBootLoaderTemplateZipl:
    def setup(self):
        self.zipl = BootLoaderTemplateZipl()

    def setup_method(self, cls):
        self.setup()

    def test_get_loader_template(self):
        assert self.zipl.get_loader_template().substitute(
            boot_timeout='10',
            bootpath='/boot',
            targetbase='',
            targettype='SCSI',
            targetblocksize='512',
            targetoffset='2048',
            targetgeometry=''
        )

    def test_get_entry_template_standard(self):
        assert self.zipl.get_entry_template().substitute(
            title='title',
            boot_options='',
            kernel_file='linux',
            initrd_file='initrd'
        )

    def test_get_entry_template_secure(self):
        assert self.zipl.get_entry_template(secure=True).substitute(
            title='title',
            boot_options='',
            secure_image_file='linux'
        )



"""
API Documentation
"""
import logging

from kiwi.logger import Logger

# Initialize custom logger class for kiwi
logging.setLoggerClass(Logger)

# Set the highest log level possible as the default log level
# in the main Logger class. This is needed to allow any logfile
# handler to log all messages by default and to allow custom log
# levels per handler. Our own implementation in Logger::setLogLevel
# will then set the log level on a handler basis
logging.getLogger('kiwi').setLevel(logging.DEBUG)



from pytest import raises

from kiwi.snapshot_manager.base import SnapshotManagerBase


class TestSnapshotManagerBase:
    def setup(self):
        self.manager = SnapshotManagerBase(
            '/dev/device', 'root_dir', 'mountpoint', '@', None
        )

    def setup_method(self, cls):
        self.setup()

    def test_create_first_snapshot(self):
        with raises(NotImplementedError):
            self.manager.create_first_snapshot()

    def test_setup_first_snapshot(self):
        with raises(NotImplementedError):
            self.manager.setup_first_snapshot()

    def test_get_default_snapshot_name(self):
        with raises(NotImplementedError):
            self.manager.get_default_snapshot_name()



from unittest.mock import (
    patch, Mock
)

from kiwi.container.wsl import ContainerImageWsl


class TestContainerImageWsl:
    def setup(self):
        self.wsl = ContainerImageWsl('root_dir')

    def setup_method(self, cls):
        self.setup()

    @patch('kiwi.container.wsl.ArchiveTar')
    @patch('kiwi.container.wsl.Compress')
    @patch('kiwi.container.wsl.Defaults.get_exclude_list_for_root_data_sync')
    @patch('kiwi.container.wsl.Defaults.get_exclude_list_from_custom_exclude_files')
    @patch('kiwi.container.wsl.Command.run')
    def test_create(
        self, mock_Command_run,
        mock_get_exclude_list_from_custom_exclude_files,
        mock_get_exclude_list_for_root_data_sync,
        mock_Compress, mock_ArchiveTar
    ):
        archive = Mock()
        mock_ArchiveTar.return_value = archive
        compress = Mock()
        mock_Compress.return_value = compress

        self.wsl.create('target_dir/image.wsl')
        mock_ArchiveTar.assert_called_once_with(
            'target_dir/image.wsl'
        )
        archive.create.assert_called_once_with(
            'root_dir',
            exclude=mock_get_exclude_list_for_root_data_sync.
            return_value + mock_get_exclude_list_from_custom_exclude_files.
            return_value,
            options=['--numeric-owner', '--absolute-names']
        )
        mock_Compress.assert_called_once_with(
            archive.create.return_value
        )
        compress.gzip.assert_called_once_with()
        mock_Command_run.assert_called_once_with(
            ['mv', compress.gzip.return_value, 'target_dir/image.wsl']
        )



from unittest.mock import (
    patch, Mock
)
from pytest import (
    fixture, raises
)

from kiwi.package_manager.apk import PackageManagerApk

from kiwi.exceptions import KiwiBootStrapPhaseFailed


class TestPackageManagerApk:
    @fixture(autouse=True)
    def inject_fixtures(self, caplog):
        self._caplog = caplog

    def setup(self):
        repository = Mock()
        repository.root_dir = '/root-dir'

        repository.runtime_config = Mock(
            return_value={
                'bootstrap_repo': 'https://some',
                'command_env': ['env']
            }
        )
        self.manager = PackageManagerApk(repository)

    def setup_method(self, cls):
        self.setup()

    def test_request_package(self):
        self.manager.request_package('name')
        assert self.manager.package_requests == ['name']

    def test_request_collection(self):
        self.manager.request_collection('name')
        assert self.manager.collection_requests == []
        assert self.manager.package_requests == ['name']

    def test_request_product(self):
        self.manager.request_product('name')
        assert self.manager.product_requests == []

    def test_request_package_exclusion(self):
        self.manager.request_package_exclusion('name')
        assert self.manager.exclude_requests == ['name']

    def test_setup_repository_modules(self):
        self.manager.setup_repository_modules({})

    def test_process_install_requests_bootstrap_raises(self):
        self.manager.bootstrap_repo = None
        with raises(KiwiBootStrapPhaseFailed):
            self.manager.process_install_requests_bootstrap()

    @patch('kiwi.command.Command.call')
    def test_process_install_requests_bootstrap_default(self, mock_call):
        self.manager.process_install_requests_bootstrap()
        mock_call.assert_called_once_with(
            [
                'apk', '--root', '/root-dir', '-X', 'https://some',
                '-U', '--allow-untrusted', '--initdb', 'add',
                'alpine-base',
            ], ['env']
        )

    @patch('kiwi.command.Command.call')
    def test_process_install_requests_bootstrap(self, mock_call):
        self.manager.request_package('vim')
        self.manager.request_collection('collection')
        self.manager.process_install_requests_bootstrap()
        mock_call.assert_called_once_with(
            [
                'apk', '--root', '/root-dir', '-X', 'https://some',
                '-U', '--allow-untrusted', '--initdb', 'add',
                'vim', 'collection'
            ], ['env']
        )

    @patch('kiwi.command.Command.call')
    @patch('kiwi.command.Command.run')
    def test_process_install_requests(self, mock_run, mock_call):
        self.manager.request_package('vim')
        self.manager.request_collection('collection')
        self.manager.request_package_exclusion('skipme')
        self.manager.process_install_requests()
        mock_run.assert_called_once_with(
            ['chroot', '/root-dir', 'apk', 'update']
        )
        mock_call.assert_called_once_with(
            ['chroot', '/root-dir', 'apk', 'add', 'vim', 'collection'],
            ['env']
        )

    @patch('kiwi.command.Command.call')
    @patch('kiwi.command.Command.run')
    def test_process_delete_requests(self, mock_run, mock_call):
        self.manager.request_package('vim')
        self.manager.process_delete_requests()
        mock_run.assert_called_once_with(
            ['chroot', '/root-dir', 'apk', 'update']
        )
        mock_call.assert_called_once_with(
            ['chroot', '/root-dir', 'apk', 'del', 'vim'],
            ['env']
        )

    def test_process_only_required(self):
        self.manager.process_only_required()
        assert self.manager.custom_args == []

    def test_process_plus_recommended(self):
        self.manager.process_plus_recommended()
        assert self.manager.custom_args == []

    @patch('kiwi.command.Command.call')
    @patch('kiwi.command.Command.run')
    def test_update(self, mock_run, mock_call):
        self.manager.update()
        mock_run.assert_called_once_with(
            ['chroot', '/root-dir', 'apk', 'update']
        )
        mock_call.assert_called_once_with(
            ['chroot', '/root-dir', 'apk', 'upgrade'],
            ['env']
        )

    def test_match_package_installed(self):
        assert self.manager.match_package_installed('foo', ' Installing foo')

    def test_match_package_deleted(self):
        assert self.manager.match_package_deleted('foo', ' Removing foo')



# Copyright (c) 2019 SUSE Linux GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
import re

# project
from kiwi.command import Command
from kiwi.defaults import Defaults
from kiwi.path import Path


class Rpm:
    """
    **Helper methods to handle the rpm database configuration**
    """
    def __init__(self, root_dir=None, macro_file=None):
        self.root_dir = root_dir
        self.config = []
        self.custom_config = []
        self.macro_path = os.sep.join(
            [
                self.root_dir or '', Defaults.get_custom_rpm_macros_path()
            ]
        )
        self.macro_file = os.sep.join(
            [
                self.macro_path,
                macro_file if macro_file else
                Defaults.get_custom_rpm_bootstrap_macro_name()
            ]
        )

    def expand_query(self, key):
        """
        Query database configuration and expand it
        """
        if self.root_dir:
            rpmdb_call = Command.run(
                ['chroot', self.root_dir, 'rpm', '-E', key]
            )
        else:
            rpmdb_call = Command.run(
                ['rpm', '-E', key]
            )
        return rpmdb_call.output.strip()

    def get_query(self, key):
        """
        Query database configuration

        Extract the first match for the given key. The method
        expects the format of entries to match the regular
        expression: '.*key[\t ]+value'

        :param string key: query key name

        :returns: match or None if there isn't any match

        :rtype: string | None
        """
        if not self.config:
            self.config = self._read_macro_configuration()
        item_list = list(
            filter(lambda item: key in item, self.config)
        )
        for item in item_list:
            match = re.match('.*{0}[\t ]+(.*)'.format(key), item)
            if match:
                return match.group(1)

    def set_config_value(self, key, value):
        """
        Set custom database configuration key/value pair

        :param string key: key name
        :param string value: key value name
        """
        self.custom_config.append('%{0}\t{1}'.format(key, value))

    def write_config(self):
        """
        Write custom database configuration

        Write bootstrap macro file to the custom rpm macros path
        """
        if self.custom_config:
            Path.create(self.macro_path)
            with open(self.macro_file, 'w') as macro:
                for entry in self.custom_config:
                    macro.write('{0}{1}'.format(entry, os.linesep))

    def wipe_config(self):
        """
        Delete custom database configuration
        """
        Path.wipe(self.macro_file)

    def _read_macro_configuration(self):
        """
        Read rpm database macro and configuration setup

        Please note we intentionally don't use 'rpm -E' here because
        it tries to expand the complete macro and if that is not
        possible it returns with the given unexpanded macro. There
        is no way to detect if the macro really existed or not.
        In addition for the use case of this class the macro should
        be read as it was configured and not its expanded form.
        """
        if self.root_dir:
            rpmdb_call = Command.run(
                ['chroot', self.root_dir, 'rpmdb', '--showrc']
            )
        else:
            rpmdb_call = Command.run(
                ['rpmdb', '--showrc']
            )
        return rpmdb_call.output.split(os.linesep)



from pytest_container.container import DerivedContainer
from .conftest import (
    CONTAINERS_WITH_ZYPPER,
    CONTAINERS_WITH_DNF
)
import pytest


VAGRANT_SETUP_CONTAINERFILE = r"""RUN groupadd vagrant && useradd -g vagrant vagrant
RUN echo $'#!/bin/bash \n\
printf "%s " "$@" >> /systemctl_params \n\
echo >> /systemctl_params \n\
'> /usr/bin/systemctl && chmod +x /usr/bin/systemctl
"""

ZYPPER_IN_CMD_CONTAINERFILE = (
    """RUN zypper -n in openssh sudo && /usr/sbin/sshd-gen-keys-start
""" + VAGRANT_SETUP_CONTAINERFILE
)


DNF_IN_CMD_CONTAINERFILE = (
    """RUN dnf -y install openssh-server && /usr/libexec/openssh/sshd-keygen ed25519
""" + VAGRANT_SETUP_CONTAINERFILE
)


@pytest.mark.parametrize(
    "container_per_test",
    [
        DerivedContainer(base=cont, containerfile=ZYPPER_IN_CMD_CONTAINERFILE)
        for cont in CONTAINERS_WITH_ZYPPER
    ] + [
        DerivedContainer(base=cont, containerfile=DNF_IN_CMD_CONTAINERFILE)
        for cont in CONTAINERS_WITH_DNF
    ],
    indirect=["container_per_test"],
)
def test_configures_system_for_vagrant(container_per_test):
    container_per_test.connection.run_expect(
        [0], ". /bin/functions.sh && baseVagrantSetup"
    )

    # check vagrant user's ssh config
    dot_ssh = container_per_test.connection.file("/home/vagrant/.ssh")
    assert dot_ssh.is_directory
    assert dot_ssh.group == "vagrant"
    assert dot_ssh.user == "vagrant"
    assert dot_ssh.mode == 0o700

    authorized_keys = container_per_test.connection.file(
        "/home/vagrant/.ssh/authorized_keys"
    )
    assert authorized_keys.is_file
    assert authorized_keys.group == "vagrant"
    assert authorized_keys.user == "vagrant"
    assert authorized_keys.mode == 0o600
    assert "vagrant insecure public key" in authorized_keys.content_string

    # check the sshd config
    sshd_config = container_per_test.connection.run_expect([0], "sshd -T").stdout
    assert "UseDNS no".lower() in sshd_config
    assert "GSSAPIAuthentication no".lower() in sshd_config

    # check that the shared /vagrant folder is present and has the correct permissions
    vagrant_shared_dir = container_per_test.connection.file("/vagrant")
    assert vagrant_shared_dir.is_directory
    assert vagrant_shared_dir.group == "vagrant"
    assert vagrant_shared_dir.user == "vagrant"

    vagrant_sudoers = container_per_test.connection.file(
        "/etc/sudoers.d/vagrant"
    )
    if vagrant_sudoers.exists and vagrant_sudoers.is_file:
        assert (
            vagrant_sudoers.content_string.strip() == "vagrant ALL=(ALL) NOPASSWD: ALL"
        )
        assert vagrant_sudoers.mode == 0o440
        assert vagrant_sudoers.user == "root"
        assert vagrant_sudoers.group == "root"
    else:
        sudoers = container_per_test.connection.file("/etc/sudoers")
        assert sudoers.exists and sudoers.is_file
        assert "vagrant ALL=(ALL) NOPASSWD: ALL" in sudoers.content_string

    # check that systemctl was called enabling sshd
    assert (
        "enable sshd"
        in container_per_test.connection.file(
            "/systemctl_params"
        ).content_string
    )



from unittest.mock import (
    patch, call, Mock
)
from pytest import raises

from kiwi.defaults import Defaults
from kiwi.storage.subformat.ova import DiskFormatOva

import kiwi

from kiwi.exceptions import (
    KiwiCommandNotFound,
    KiwiFormatSetupError,
    KiwiTemplateError
)


class TestDiskFormatOva:
    @patch('kiwi.storage.subformat.ova.FirmWare')
    def setup(self, mock_FirmWare):
        self.firmware = Mock()
        self.firmware.efi_mode = Mock(
            return_value='uefi'
        )
        mock_FirmWare.return_value = self.firmware
        Defaults.set_platform_name('x86_64')
        self.context_manager_mock = Mock()
        self.file_mock = Mock()
        self.enter_mock = Mock()
        self.exit_mock = Mock()
        self.enter_mock.return_value = self.file_mock
        setattr(self.context_manager_mock, '__enter__', self.enter_mock)
        setattr(self.context_manager_mock, '__exit__', self.exit_mock)
        xml_data = Mock()
        xml_data.get_name = Mock(
            return_value='some-disk-image'
        )
        xml_data.get_displayname = Mock(
            return_value=None
        )
        self.xml_state = Mock()
        self.xml_state.xml_data = xml_data
        self.xml_state.get_image_version = Mock(
            return_value='1.2.3'
        )

        self.machine_setup = Mock()
        self.xml_state.get_build_type_machine_section = Mock(
            return_value=self.machine_setup
        )
        self.machine_setup.get_HWversion = Mock(
            return_value='42'
        )
        self.machine_setup.get_guestOS = Mock(
            return_value='suse'
        )
        self.machine_setup.get_memory = Mock(
            return_value='4096'
        )
        self.machine_setup.get_ncpus = Mock(
            return_value='2'
        )

        self.iso_setup = Mock()
        self.xml_state.get_build_type_vmdvd_section = Mock(
            return_value=self.iso_setup
        )
        self.xml_state.get_luks_credentials = Mock(
            return_value=None
        )
        self.iso_setup.get_controller = Mock(
            return_value='ide'
        )
        self.iso_setup.get_id = Mock(
            return_value='0'
        )

        self.network_setup = [Mock()]
        self.xml_state.get_build_type_vmnic_entries = Mock(
            return_value=self.network_setup
        )
        self.network_setup[0].get_interface = Mock(
            return_value='0'
        )
        self.network_setup[0].get_mac = Mock(
            return_value='98:90:96:a0:3c:58'
        )
        self.network_setup[0].get_mode = Mock(
            return_value='bridged'
        )
        self.network_setup[0].get_driver = Mock(
            return_value='e1000'
        )

        self.disk_setup = Mock()
        self.xml_state.get_build_type_vmdisk_section = Mock(
            return_value=self.disk_setup
        )
        self.disk_setup.get_controller = Mock(
            return_value='lsilogic'
        )
        self.disk_setup.get_id = Mock(
            return_value='0'
        )

        self.machine_setup.get_ovftype = Mock(
            return_value='vmware'
        )

        self.runtime_config = Mock()
        self.runtime_config.get_bundle_compression.return_value = False
        kiwi.storage.subformat.base.RuntimeConfig = Mock(
            return_value=self.runtime_config
        )

        self.disk_format = DiskFormatOva(
            self.xml_state, 'root_dir', 'target_dir'
        )

    def setup_method(self, cls):
        self.setup()

    @patch('kiwi.storage.subformat.ova.FirmWare')
    def test_post_init(self, mock_FirmWare):
        self.disk_format.post_init({})

    def test_post_init_bad_ovftype(self):
        self.machine_setup.get_ovftype.return_value = 'foobar'
        with raises(KiwiFormatSetupError):
            self.disk_format.post_init({})

    def test_store_to_result(self):
        result = Mock()
        self.disk_format.store_to_result(result)
        assert result.add.call_args_list == [
            call(
                compress=False,
                filename='target_dir/some-disk-image.x86_64-1.2.3.ova',
                key='disk_format_image',
                shasum=True,
                use_for_bundle=True
            )
        ]

    @patch('kiwi.storage.subformat.ova.Path.which')
    @patch('kiwi.storage.subformat.ova.Command.run')
    @patch('kiwi.storage.subformat.ova.DiskFormatVmdk.create_image_format')
    @patch('os.chdir')
    def test_create_image_format(
        self,
        mock_chdir,
        mock_create_image_format,
        mock_command,
        mock_which
    ):
        mock_which.return_value = 'ova-compose'
        with patch('builtins.open', create=True) as mock_open:
            self.disk_format.create_image_format()
            mock_open.assert_called_once_with(
                'target_dir/some-disk-image.x86_64-1.2.3.meta', 'w'
            )
            mock_command.assert_called_once_with(
                [
                    'ova-compose',
                    '--input-file',
                    'target_dir/some-disk-image.x86_64-1.2.3.meta',
                    '--output-file',
                    'target_dir/some-disk-image.x86_64-1.2.3.ova',
                    '--format', 'ova'
                ]
            )

    @patch('kiwi.storage.subformat.ova.DiskFormatVmdk.create_image_format')
    @patch('kiwi.storage.subformat.ova.OvaComposeTemplate.get_template')
    @patch('kiwi.storage.subformat.ova.Command.run')
    @patch('kiwi.storage.subformat.ova.Path.which')
    def test_create_image_format_template_error(
        self,
        mock_Path_which,
        mock_Command_run,
        mock_OvaComposeTemplate_get_template,
        mock_DiskFormatVmdk_create_image_format
    ):
        template = Mock()
        mock_Path_which.return_value = 'ova-compose'
        mock_OvaComposeTemplate_get_template.return_value = template
        template.substitute.side_effect = Exception
        with patch('builtins.open'):
            with raises(KiwiTemplateError):
                self.disk_format.create_image_format()

    @patch('kiwi.storage.subformat.ova.Path.which')
    def test_create_image_format_no_ova_compose(self, mock_Path_which):
        mock_Path_which.return_value = None
        with raises(KiwiCommandNotFound):
            self.disk_format.create_image_format()



# Copyright (c) 2020 SUSE Software Solutions Germany GmbH.  All rights reserved.
#
# This file is part of kiwi.
#
# kiwi is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kiwi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kiwi.  If not, see <http://www.gnu.org/licenses/>
#
import os
from lxml import etree
from xml.dom import minidom

# project
from kiwi.container.setup.base import ContainerSetupBase
from kiwi.exceptions import KiwiContainerSetupError
from kiwi.defaults import Defaults


class ContainerSetupAppx(ContainerSetupBase):
    """
    WSL/Appx container setup
    """
    def setup(self):
        """
        Setup container metadata
        """
        self.appx_at_microsoft = 'http://schemas.microsoft.com/appx'
        self.appx_manifest_file = os.sep.join(
            [self.custom_args.get('metadata_path'), 'AppxManifest.xml']
        )
        self.appx_namespace_map = dict(
            wsl='{0}/manifest/foundation/windows10'.format(
                self.appx_at_microsoft
            ),
            uap='{0}/manifest/uap/windows10'.format(
                self.appx_at_microsoft
            ),
            desktop='{0}/manifest/desktop/windows10'.format(
                self.appx_at_microsoft
            ),
            uap3='{0}/manifest/uap/windows10/3'.format(
                self.appx_at_microsoft
            ),
            mp='{0}/2014/phone/manifest'.format(
                self.appx_at_microsoft
            )
        )
        if not os.path.exists(self.appx_manifest_file):
            raise KiwiContainerSetupError(
                'Appx Manifest not found at: {0}'.format(
                    self.appx_manifest_file
                )
            )
        try:
            self.appx_manifest = etree.parse(
                self.appx_manifest_file
            )
            self._update_identity()
            self._update_desktop_visual()
            self._update_application_metadata()
            xml_data_unformatted = etree.tostring(
                self.appx_manifest.getroot()
            )
            xml_data_domtree = minidom.parseString(xml_data_unformatted)
            xml_data_raw = xml_data_domtree.toprettyxml(
                indent='    ', encoding="utf-8"
            )
            with open(self.appx_manifest_file, 'wb') as appx_manifest:
                for line in xml_data_raw.split(b'\n'):
                    if line.split():
                        appx_manifest.write(line + b'\n')
        except Exception as issue:
            raise KiwiContainerSetupError(
                'Failed to update Appx Manifest: {0}'.format(issue)
            )

    def _update_identity(self):
        identities = self._get_xpath('//wsl:Identity')
        history = self.custom_args.get('history') or {}
        for identity in identities:
            if self._get_microsoft_processor_architecture():
                identity.set(
                    'ProcessorArchitecture',
                    self._get_microsoft_processor_architecture()
                )
            if history.get('package_version'):
                identity.set('Version', history.get('package_version'))
            if history.get('created_by'):
                identity.set('Publisher', history.get('created_by'))

    def _update_desktop_visual(self):
        history = self.custom_args.get('history') or {}
        visuals = self._get_xpath(
            '//wsl:Applications/wsl:Application/uap:VisualElements'
        )
        for visual in visuals:
            if history.get('comment'):
                visual.set('Description', history.get('comment'))

    def _update_application_metadata(self):
        history = self.custom_args.get('history') or {}
        applications = self._get_xpath('//wsl:Applications/wsl:Application')
        extensions = self._get_xpath(
            '//wsl:Applications/wsl:Application/'
            'wsl:Extensions/uap3:Extension'
        )
        aliases = self._get_xpath(
            '//wsl:Applications/wsl:Application/'
            'wsl:Extensions/uap3:Extension/uap3:AppExecutionAlias/'
            'desktop:ExecutionAlias'
        )
        for application in applications:
            if history.get('application_id'):
                application.set('Id', history.get('application_id'))
            if history.get('launcher'):
                application.set('Executable', history.get('launcher'))
        for extension in extensions:
            if history.get('launcher'):
                extension.set('Executable', history.get('launcher'))
        for alias in aliases:
            if history.get('launcher'):
                alias.set('Alias', history.get('launcher'))

    def _get_xpath(self, expression):
        return self.appx_manifest.getroot().xpath(
            expression, namespaces=self.appx_namespace_map
        )

    def _get_microsoft_processor_architecture(self):
        windows = {
            'x86_64': 'x64'
        }
        return windows.get(Defaults.get_platform_name())



from pytest import raises
import os

from kiwi.utils.toenv import ToEnv
from kiwi.exceptions import KiwiEnvImportError


class TestToEnv(object):
    def setup(self):
        ToEnv('../data', 'some.env')
        assert os.environ['ZYPP_MODALIAS_SYSFS'] == ''

    def setup_method(self, cls):
        self.setup()

    def test_setup_raises(self):
        with raises(KiwiEnvImportError):
            ToEnv('../data', 'some_broken.env')
